@@ -5,7 +5,7 @@ set.seed(1, kind="Mersenne-Twister", normal.kind="Inversion")
 
 test_that("prior predictive sampling works", {
   df <- data.frame(int=rep(1, 1000))
-  dat <- generate_data(~ reg(~ 1, b0=2, Q0=1e10), sigma.mod=pr_invchisq(df=1e6, scale=4), data=df)
+  dat <- generate_data(~ reg(~ 1, b0=2, Q0=1e10), sigma.mod=pr_fixed(value=4), data=df)
   expect_true((1.5 < mean(dat$y) && mean(dat$y) < 2.5))
   expect_true((1.5 < sd(dat$y) && sd(dat$y) < 2.5))
 })
